%% Szerző:  Páter-Részeg Attila
%% NEPTUN:  RD1SET
%% Eseménykezelés Erlangban

%% Használt típusok:
%% szerzo() = string() | atom()
%% konyvcim() = string() | atom()
%% darabszam() = integer()

-module(konyvtar).
-export([start/1]).
-export([uj_konyv/2, kolcsonoz/4, visszahoz/4]).
-export([listaz_kolcsonozheto/1, listaz_kolcsonzott/1, listaz_osszes/1]).

%% start(list()) -> {ok, Pid}
%% Paraméterként egy listában felsorolva megadhatjuk a vezérlőhöz adandó eseménykezelőket. A példában üres lista esetén, vagy ha nem 
%% létező modulnevet adunk meg, az adatbazis modul kerül hozzáadásra.
%% Használat (példa): 
%% konyvtar:start([adatbazis]).
start(Kezelok) ->

    % Létrejön az Event Manager (vezérlő)
    {ok, Pid} = gen_event:start_link(),

    % Hozzáadjuk a paraméterben kapott kezelőket, ha nem sikerül, akkor hozzáadjuk az adatbazis modult
    case [gen_event:add_handler(Pid, Elem, []) || Elem <- Kezelok] of
        [] -> gen_event:add_handler(Pid, adatbazis, []);
        [{'EXIT', _}] -> gen_event:add_handler(Pid, adatbazis, []);
        _ -> ok
    end,

    % beolvasás esemény kiváltása
    beolvasas(Pid),
    {ok, Pid}.

%% uj_konyv(Pid, {szerzo(), konyvcim(), darabszam()}) -> string()
%% Új könyv hozzáadása esemény kiváltása (kezelőspecifikus kéréssel)
uj_konyv(Pid, Konyv) ->
    E = gen_event:call(Pid, adatbazis, {uj_konyv, Konyv}),
    E.

%% kolcsonoz(Pid, szerzo(), konyvcim(), darabszam()) -> ok
%% Meglévő könyv kikölcsönzését kérhetjük. A sikerességről üzenetet kapunk a standard outputra.
kolcsonoz(Pid, Szerzo, KonyvCim, Db) ->
    gen_event:notify(Pid, {kolcsonoz, Szerzo, KonyvCim, Db}).

%% visszahoz(Pid, szerzo(), konyvcim(), darabszam()) -> ok
%% Kint könyv visszahozatala. A sikerességről üzenetet kapunk a standard outputra.
visszahoz(Pid, Szerzo, KonyvCim, Db) ->
    gen_event:notify(Pid, {visszahoz, Szerzo, KonyvCim, Db}).

%% listaz_kolcsonozheto(Pid) -> {atomic, list()}
%% A kölcsönözhető könyvek kilistázását kérhetjük.
listaz_kolcsonozheto(Pid) ->
    gen_event:call(Pid, adatbazis, in).

%% listaz_kolcsonzott(Pid) -> {atomic, list()}
%% A kölcsönzött könyvek kilistázását kérhetjük.
listaz_kolcsonzott(Pid) ->
    gen_event:call(Pid, adatbazis, out).

%% listaz_osszes(Pid) -> {atomic, list()}
%% Az összes könyv kilistázását kérhetjük.
listaz_osszes(Pid) ->
    gen_event:call(Pid, adatbazis, all).

%% beolvasas(Pid) -> string()
%% Az adatbázis könyveinek beolvasása (jelen esetben nem csinál semmit, csak kiírja, hogy sikeres volt, ugyanis az 
%% adatbázis létrejöttével elérhetők a könyvek)
beolvasas(Pid) ->
    E = gen_event:call(Pid, adatbazis, beolvasas),
    E.